#include <avr/io.h>
#include "MMC_SD/MMC_SD.h"
#include "FAT/FAT.h"
#include <util/delay.h>

// control line pins
#define	LCD_CD	PC7   
#define	LCD_WR  PC6
#define	LCD_RD  PC5
#define	LCD_CS	PC4
#define	LCD_RES	PC2

// macros for control lines
#define CLR_CD PORTC &= ~(1<<LCD_CD);
#define SET_CD PORTC |=  (1<<LCD_CD);

#define CLR_CS PORTC &= ~(1<<LCD_CS);
#define SET_CS PORTC |=  (1<<LCD_CS);

#define CLR_RESET PORTC &= ~(1<<LCD_RES);
#define SET_RESET PORTC |=  (1<<LCD_RES);

#define CLR_WR PORTC &= ~(1 << LCD_WR);
#define SET_WR PORTC |=  (1 << LCD_WR);
#define CLR_RD PORTC &= ~(1 << LCD_RD);
#define SET_RD PORTC |=  (1 << LCD_RD);

// color definitions
#define	BLACK	  0x0000
#define	BLUE	  0x001F
#define	RED 	  0xF800
#define	GREEN   0x07E0
#define CYAN	  0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE	  0xFFFF

#define uint8 unsigned char
#define uint16 unsigned int
#define uint32 unsigned long

unsigned char * PATH = "\\batch";

extern uint16 SectorsPerClust;
extern uint16 FirstDataSector; 
extern uint8 FAT32_Enable;

struct FileInfoStruct FileInfo;

struct direntry PictureInfo;
uint16 totalfiles = 0;

/*************************************************/
void delay(unsigned int t)
{
	unsigned char t1;
	while(t--)
	for(t1=11;t1>0;t1--)
	{asm("nop");}
}
/*************************************************/
void write_command(unsigned int command)
{
  CLR_CS;   // chip selected
  CLR_CD;   // command mode

	SPI_WriteByte(command);	// set up data on bus

  SET_CS;   // deselect chip
}

/*************************************************/
void write_data(unsigned int data)
{
  SET_CD;       // data mode
  CLR_CS;       // chip selected

	SPI_WriteByte(data);	// set up data on bus

  SET_CS;       // deselect chip
}
/*************************************************/
void initialization()
{
	CLR_RESET;
	delay(200);
	SET_RESET;
	delay(500);

	write_command(0xAE);    //  10101111:  set display off
  write_command(0xA4);    //  10100100:  set nomal display(POR)

	write_command(0x15);    //  00010101:  Set Column address
	write_command(0x00);    //  00000000:  A[6:0] sets the column start address from 0-95, RESET=00d.
	write_command(0x5F);    //  01011111:  B[6:0] sets the column end address from 0-95 RESET=95d.

	write_command(0x75);    //  10000001:  Set Row Address
	write_command(0x00);    //  00000000:  A[5:0] sets the row start address from 0-63, RESET=00d.
	write_command(0x3F);    //  00111111:  B[5:0] sets the row end address from 0-63, RESET=63d.

	write_command(0x87);    //  10000111:  Master current Control
	write_command(0x07);    //  00000111:  00000000 to 00001111 for attenuation factor (less is more)

	write_command(0x81);    //  10000001:  set contrast control for color A
	write_command(0xA0);    //  11111111:  contrast set(select from 1 to 256)

	write_command(0x82);    //  10000010:  set contrast control for color B
	write_command(0x60);    //  11111111:  contrast set(select from 1 to 256)

	write_command(0x83);    //  10000011:  set contrast control for color C
	write_command(0xB0);    //  11111111:  contrast set(select from 1 to 256)

	write_command(0xA0);    //  10100000:  set re-map & data format
	write_command(0x60);    //  01100000:  a[7,6]:01:65k color format(por)CCCCCBBBBBBAAAAA   00:256 color formatCCCBBBAA

	write_command(0xA1);	  //  10100001:  set display start line
	write_command(0x00);
	
	write_command(0xA2);    //  10100010:  set display offset
	write_command(0x00);
	
	write_command(0xA8);	  //  10101000:  set multiplex ratio
	write_command(0x3F);
	
	write_command(0xB0);    //  set power save
	write_command(0x00);    //  0x12:power saving mode
	
	write_command(0xB1);		//  phase 1 and period adjustment(discharge,charge)
	write_command(0x74);
	
	write_command(0xB3);		//  display clock divider/oscillator frequency
	write_command(0xD0);

	write_command(0xBB);		//  set vpa 
	write_command(0x1F);

	write_command(0xBC);		//  set vpb
	write_command(0x1F);

	write_command(0xBD);    //  set vpc
	write_command(0x1F);		
	
	write_command(0xBE);		//  set vcomh
	write_command(0x3F); 

	write_command(0xAD);    //  10101101:  set contrast control for color A
	write_command(0x8E);    //  11111111:  contrast set(select from 1 to 256)

  write_command(0xAF);    //  display on
}
/*************************************************/
void Display_Home()
{
	write_command(0x15);    //  00010101:  Set Column address
	write_command(0x00);    //  00000000:  A[6:0] sets the column start address from 0-95, RESET=00d.
	write_command(0x5F);    //  01011111:  B[6:0] sets the column end address from 0-95 RESET=95d.

	write_command(0x75);    //  10000001:  Set Row Address
	write_command(0x00);    //  00000000:  A[5:0] sets the row start address from 0-63, RESET=00d.
	write_command(0x3F);    //  00111111:  B[5:0] sets the row end address from 0-63, RESET=63d.
}
/*************************************************/
void display_rgb(unsigned int data)
{
	Display_Home();

// white, red, green, blue screen
	int i,j;
	for(i=0;i<64;i++)
	{
		for(j=0;j<96;j++)
		{
			if(j>=0 && j<24)
			{
				write_data(0xff);
				write_data(0xff);		//white display
			}
			if(j>23 && j<48)
			{
				write_data(0x00);
				write_data(0x1f);		//blue display
			}
			if(j>47 && j<72)
			{
				write_data(0x07);		//green display
				write_data(0xe0);
			}
			if(j>71 && j<96)
			{
				write_data(0xf8);		//red display
				write_data(0x00);
			}
		}
	}
}
/*************************************************/
void Picture_Slide_Show()
{
	uint8 i;              //loop variable
	uint16 j;             //loop variable
	uint32 p;             //cluster
	uint16 *buffer;				//buffer
	uint16 pics = 1;
	uint16 slide_show_flag = 1;
	uint8 sector;
	uint32 pixels;

	if (totalfiles == 0)
	return;

	buffer = malloc(512);
	if (buffer == 0)
	return;

next: 	//label for "goto"

	Search(PATH, &PictureInfo, &pics);                                         //find the file
	p = PictureInfo.deStartCluster + (((uint32)PictureInfo.deHighClust) << 16);//the first cluster of the file

	sector = 0;

	FAT_LoadPartCluster(p, sector, buffer);	//read a sector

	pixels = (uint32) 96 * 64 + 1; // total # of pixels to write

	Display_Home();
	j = 0; // byte count
	
	while (1)
	{
		write_data(buffer[j]);  // write 8 bits
		write_data(buffer[j]>>8);  // write 8 bits
		pixels--;               // which is one pixel
		j++;                    // word count

		if (pixels == 0)        
		break;

		if (j == 256)           // time for a new sector     
		{
			sector++;
			if (sector == SectorsPerClust)
			{
				p = FAT_NextCluster(p);				// read next cluster
				sector = 0;
			}

			FAT_LoadPartCluster(p, sector, buffer);	// read a sector
			j = 0;
		}
	}

	if (slide_show_flag)
	{
		for (i=0;i<15;i++)	// delay for a while
		delay(0xffff);

		pics++;					// increment picture number
		if (pics > totalfiles)	// if last
		pics = 1;			// wrap around

		goto next;				// go show next pic
	}
	free(buffer);
}
/*************************************************/
void LCD_test(char color)
{
	int i,j;

	write_command(0x15);    //  00010101:  Set Column address
	write_command(0x00);    //  00000000:  A[6:0] sets the column start address from 0-95, RESET=00d.
	write_command(0x5F);    //  01011111:  B[6:0] sets the column end address from 0-95 RESET=95d.

	write_command(0x75);    //  10000001:  Set Row Address
	write_command(0x00);    //  00000000:  A[5:0] sets the row start address from 0-63, RESET=00d.
	write_command(0x3F);    //  00111111:  B[5:0] sets the row end address from 0-63, RESET=63d.

	for(i=0;i<64;i++)
	{
		for(j=0;j<=96;j++)
		{
		  if(color==0)
			{
				write_data(0xff);
				write_data(0xff);		//white display
			}
		  if(color==1)
			{
				write_data(0x00);
				write_data(0x1f);		//blue display
			}
		  if(color==2)
			{
				write_data(0x07);		//green display
				write_data(0xe0);
			}
		  if(color==3)
			{
				write_data(0xf8);		//red display
				write_data(0x00);
			}		
		  if(color==4)
			{
				write_data(0x00);		//black display
				write_data(0x00);
			}
		}       
	}
}
/*************************************************/
int main()
{
	DDRC  = 0xFE;	    

 	PORTC = 0xCE;

	uint8 retry = 0;

	OSCCAL = 0xFF;    

	delay(0xffff);

	MMC_SD_Init();    //SPI initialize

	// Initialize the display
	initialization();

	sd_restart: 	//label for "goto"

	while (MMC_SD_Reset())  // Initialize SD Card
	{
		retry++;
		if (retry > 254)
		{
			while (1)
			{
				goto sd_restart;				// try again
			}
		}
	}

	delay(0xffff);      

	if (FAT_Init())   	//initialize file system - FAT16 and FAT32 are supported
	{
		while (1)
		{
			goto sd_restart;				// try again
		}
	}

	Search(PATH, &PictureInfo, &totalfiles);

	Picture_Slide_Show();

	while (1)
	{
	}
	return 0;

}
/*************************************************/


